package FIPSO;

/*
	particle.java
	Pablo David Gutiérrez Pérez.
	
	Copyright (c) 2011.

*/

import java.util.ArrayList;
import java.util.Random;

/**
 *
 * @author Pablo David Gutiérrez Pérez
 */

public class particle {
	/** Cognitive knowledge factor */
	private static double cognitive = 2.05;
	/** Social knowledge factor */
	private static double social = 2.05;
	/** Constriction factor value */
	private static double constriction = 0.7298;
	
	/** Problem's dimension */
	protected int dimension;
	/** Actual position */
	protected double[] position;
	/** best position factor */
	protected double[] best_position;
	/** Velocity vector */
	protected double[] velocity;
	/** Problem bounds */
	protected double max_bounds;
	protected double min_bounds;
	/** Maximum velocity on a component */
	protected double max_velocity;
	/** Fitness of actual position */
	protected double fitness;
	/** Fitness of best position */
	protected double best_fitness;

	/** Default constructor empty */
	public particle(){}
	
	/** Particle constructor
	 * @param dim Problem's dimension
	 * @param max High bound for each dimension
	 * @param min Low bound for each dimension
	 */
	public particle(int dim, double max, double min){
		dimension = dim;
		position = new double[dimension];
		best_position = new double[dimension];
		velocity = new double[dimension];
		max_bounds = max;
		min_bounds = min;
		max_velocity = 0.2*Math.abs(max_bounds - min_bounds);
	}

	/** Copy constructor, makes a new particle equal to another given
	 * @param base Particle to copy.
	 */
	public particle(particle base){
		dimension = base.dimension;
		position = base.position.clone();
		best_position = base.best_position.clone();
		velocity = base.velocity.clone();
		max_bounds = base.max_bounds;
		min_bounds = base.min_bounds;
		max_velocity = base.max_velocity;
	}

	/** Invert the position and velocity of a particle
	 * @param max High bound.
	 * @param min Low bound.
	 */
	public void invert(double max, double min){
		double mid = (max - min) / 2;
		double dif;
		for(int i = 0; i < position.length; i++){
			dif = Math.abs(mid - position[i]);
			if(position[i] < mid)
				position[i] = mid + dif;
			else
				position[i] = mid - dif;
			best_position[i] = position[i];
		}
		for(int i = 0; i < velocity.length; i++)
			velocity[i] = -1.0 * velocity[i];
	}
	
	/** Initializes a particle position and velocity
	 * @param rand Random generator.
	 */
	public void init(Random rand){
		for(int i = 0; i < dimension; i++){
			position[i] = rand.nextDouble()*(max_bounds - min_bounds) + min_bounds;
			best_position[i] = position[i];
			velocity[i] = rand.nextDouble()*(max_velocity * 2) - max_velocity;
		}
	}

	/** Calculate the reference position to update the velocity
	 * @param environment Environment.
	 * @param rand Random generator.
	 * @param type Weight model used.
	 * @param topology topology used.
	 * @param best Best particle.
	 * @param worse Worse particle.
	 */
	private double[] reference_position(ArrayList<particle> environment, Random rand, int type, int topology, double best,double worse){
		if(topology < 5 || type >=2)
			environment.add(this);

		double ref[] = new double[dimension];
		double coef[] = new double[environment.size()];
		double w[] = new double[environment.size()];
		
		for(int i = 0; i < ref.length; i++)
			ref[i] = 0;
		
		/** Generate the random coefficients  */
		for(int i = 0; i < environment.size(); i++)
			coef[i] = rand.nextDouble()*((social+cognitive)/environment.size());
		
		//Calc w
		double wor = environment.get(0).best_fitness;
		double bes = environment.get(0).best_fitness;
		double aux = 0;
		switch(type){
			// Every particle on environment has the same weight
			case 0:
				for(int i = 0; i < environment.size(); i++)
					w[i] = 1.0;
				break;
			
			// Lineal best-worse model
			case 1:
				for(int i = 1; i < environment.size(); i++){
					if(environment.get(i).best_fitness > wor)
						wor = environment.get(i).best_fitness;
					if(environment.get(i).best_fitness < bes)
						bes = environment.get(i).best_fitness;
				}
				for(int i = 0; i < environment.size(); i++)
					w[i] = (1-((environment.get(i).best_fitness - best)/(worse-best)));
				break;
				
			// Every particle has the same weitght except the own particle which has the half of it
			case 2:
				for(int i = 0; i < environment.size()-1; i++){
					w[i] = 0.5;
					aux += w[i];
				}
				w[environment.size()-1] = aux;
				break;
				
			//Lineal best-worse model whit own particle having the half of the weight
			case 3:
				for(int i = 1; i < environment.size(); i++){
					if(environment.get(i).best_fitness > wor)
						wor = environment.get(i).best_fitness;
					if(environment.get(i).best_fitness < bes)
						bes = environment.get(i).best_fitness;
				}
				for(int i = 0; i < environment.size(); i++){
					w[i] = (1-((environment.get(i).best_fitness - best)/(worse-best)));
					aux += w[i];
				}
				w[environment.size()-1] = aux;
				break;
		}
		
		double div = 0;
		for(int i = 0; i < environment.size(); i++){
			for(int j = 0; j < ref.length; j++)
				ref[j] += w[i]*coef[i]*environment.get(i).best_position[j];
			div += w[i]*coef[i];
		}
		for(int i = 0; i < ref.length; i++)
			ref[i] = ref[i]/div;
		
		return ref;
	}
	
	/** Updates particle's velocity
	 * @param global_best reference particle.
	 * @param rand Random generator.
	 * @param type Weight model used.
	 * @param topology topology used.
	 * @param best Best particle.
	 * @param worse Worse particle.
	 */
	public void update_velocity(ArrayList<particle> environment, Random rand, int type, int topology, double best, double worse){
		double reference[];
		reference = reference_position(environment, rand, type, topology, best, worse);
		for(int i = 0; i < dimension; i++){
			velocity[i] = constriction * (velocity[i] + ((cognitive + social)*(reference[i] - position[i])));
			if(velocity[i] < -max_velocity)
				velocity[i] = -max_velocity;
			else if(velocity[i] > max_velocity)
				velocity[i] = max_velocity;
		}
	}

	/** Updates particle's position
	 */
	public void update_position(){
		for( int i = 0; i < dimension; i++){
			position[i] = position[i] + velocity[i];
			if(position[i] > max_bounds){
				position[i] = max_bounds;
				velocity[i] = 0;
			}
			else if(position[i] < min_bounds)
			{
				position[i] = min_bounds;
				velocity[i] = 0;
			}
		}
	}

	/** Prints information about the particle
	 * @param n number of particle
	 */
	public void print(int n){
		System.out.println("particle: "+n);
		for(int i = 0; i < position.length; i++){
			System.out.println(position[i]+","+velocity[i]+","+best_position[i]);
		}
		System.out.println(best_fitness + "," + fitness);
	}
};
